"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _initializerDefineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/initializerDefineProperty"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _applyDecoratedDescriptor2 = _interopRequireDefault(require("@babel/runtime/helpers/applyDecoratedDescriptor"));

var _initializerWarningHelper2 = _interopRequireDefault(require("@babel/runtime/helpers/initializerWarningHelper"));

var _l7Utils = require("@antv/l7-utils");

var _inversify = require("inversify");

require("reflect-metadata");

var _types = require("../../types");

var _clock = _interopRequireDefault(require("../../utils/clock"));

var _dec, _dec2, _dec3, _dec4, _class, _class2, _descriptor, _descriptor2, _descriptor3;

var LayerService = (_dec = (0, _inversify.injectable)(), _dec2 = (0, _inversify.inject)(_types.TYPES.IRendererService), _dec3 = (0, _inversify.inject)(_types.TYPES.IMapService), _dec4 = (0, _inversify.inject)(_types.TYPES.IGlobalConfigService), _dec(_class = (_class2 = function () {
  function LayerService() {
    (0, _classCallCheck2.default)(this, LayerService);
    (0, _defineProperty2.default)(this, "clock", new _clock.default());
    (0, _defineProperty2.default)(this, "alreadyInRendering", false);
    (0, _defineProperty2.default)(this, "layers", []);
    (0, _defineProperty2.default)(this, "layerRenderID", void 0);
    (0, _defineProperty2.default)(this, "sceneInited", false);
    (0, _defineProperty2.default)(this, "animateInstanceCount", 0);
    (0, _defineProperty2.default)(this, "lastRenderType", void 0);
    (0, _defineProperty2.default)(this, "lastPickRenderTime", void 0);
    (0, _initializerDefineProperty2.default)(this, "renderService", _descriptor, this);
    (0, _initializerDefineProperty2.default)(this, "mapService", _descriptor2, this);
    (0, _initializerDefineProperty2.default)(this, "configService", _descriptor3, this);
  }

  (0, _createClass2.default)(LayerService, [{
    key: "add",
    value: function add(layer) {
      if (this.sceneInited) {
        layer.init();
      }

      this.layers.push(layer);
    }
  }, {
    key: "initLayers",
    value: function initLayers() {
      this.sceneInited = true;
      this.layers.forEach(function (layer) {
        if (!layer.inited) {
          layer.init();
        }
      });
    }
  }, {
    key: "getLayers",
    value: function getLayers() {
      return this.layers;
    }
  }, {
    key: "getLayer",
    value: function getLayer(id) {
      return this.layers.find(function (layer) {
        return layer.id === id;
      });
    }
  }, {
    key: "getLayerByName",
    value: function getLayerByName(name) {
      return this.layers.find(function (layer) {
        return layer.name === name;
      });
    }
  }, {
    key: "remove",
    value: function remove(layer) {
      var layerIndex = this.layers.indexOf(layer);

      if (layerIndex > -1) {
        this.layers.splice(layerIndex, 1);
      }

      layer.emit('remove', null);
      layer.destroy();
      this.renderLayers();
    }
  }, {
    key: "removeAllLayers",
    value: function removeAllLayers() {
      this.destroy();
    }
  }, {
    key: "renderLayers",
    value: function renderLayers(renderType) {
      if (!this.renderTest(renderType)) {
        return;
      }

      if (this.alreadyInRendering) {
        return;
      }

      this.alreadyInRendering = true;
      this.clear();
      this.updateRenderOrder();
      this.layers.filter(function (layer) {
        return layer.inited;
      }).filter(function (layer) {
        return layer.isVisible();
      }).forEach(function (layer) {
        layer.hooks.beforeRenderData.call();
        layer.hooks.beforeRender.call();
        layer.render();
        layer.hooks.afterRender.call();
      });
      this.alreadyInRendering = false;
    }
  }, {
    key: "updateRenderOrder",
    value: function updateRenderOrder() {
      this.layers.sort(function (pre, next) {
        return pre.zIndex - next.zIndex;
      });
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.layers.forEach(function (layer) {
        return layer.destroy();
      });
      this.layers = [];
      this.renderLayers();
    }
  }, {
    key: "startAnimate",
    value: function startAnimate() {
      if (this.animateInstanceCount++ === 0) {
        this.clock.start();
        this.runRender();
      }
    }
  }, {
    key: "stopAnimate",
    value: function stopAnimate() {
      if (--this.animateInstanceCount === 0) {
        this.stopRender();
        this.clock.stop();
      }
    }
  }, {
    key: "getOESTextureFloat",
    value: function getOESTextureFloat() {
      return this.renderService.extensionObject.OES_texture_float;
    }
  }, {
    key: "renderTest",
    value: function renderTest(renderType) {
      var now = new Date().getTime();
      var betweenPickRenderTime = now - this.lastPickRenderTime;

      if (renderType === 'picking') {
        this.lastPickRenderTime = new Date().getTime();
      }

      if (renderType) {
        switch (renderType) {
          case 'picking':
            if (this.lastRenderType === 'mapRender' || this.lastRenderType === 'animate') {
              this.lastRenderType = 'picking';

              if (betweenPickRenderTime > 48) {
                return true;
              } else {
                return false;
              }
            } else {
              this.lastRenderType = 'picking';
              return true;
            }

          case 'animate':
            if (this.lastRenderType === 'mapRender') {
              this.lastRenderType = 'animate';
              return false;
            } else {
              this.lastRenderType = 'animate';
              return true;
            }

          case 'mapRender':
            this.lastRenderType = 'mapRender';
            return true;

          default:
            return true;
        }
      }

      return true;
    }
  }, {
    key: "clear",
    value: function clear() {
      var color = (0, _l7Utils.rgb2arr)(this.mapService.bgColor);
      this.renderService.clear({
        color: color,
        depth: 1,
        stencil: 0,
        framebuffer: null
      });
    }
  }, {
    key: "runRender",
    value: function runRender() {
      this.renderLayers('animate');
      this.layerRenderID = requestAnimationFrame(this.runRender.bind(this));
    }
  }, {
    key: "stopRender",
    value: function stopRender() {
      cancelAnimationFrame(this.layerRenderID);
    }
  }]);
  return LayerService;
}(), (_descriptor = (0, _applyDecoratedDescriptor2.default)(_class2.prototype, "renderService", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor2 = (0, _applyDecoratedDescriptor2.default)(_class2.prototype, "mapService", [_dec3], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor3 = (0, _applyDecoratedDescriptor2.default)(_class2.prototype, "configService", [_dec4], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class2)) || _class);
exports.default = LayerService;
//# sourceMappingURL=LayerService.js.map